/****************************************************************************
** mod_template.cc
**
** Template for plugins
**
** Author: Author Name <email@address>
**
** Copyright (c) 2000 Mount Linux Inc.
**
** This file is part of Mount Linux Olympus.
**
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 2 as published
** by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
** or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
** for more details.
**
** You should have received a copy of version 2 of the GNU General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*****************************************************************************/

#include "mod_template.h"
#include "guisocket.h"

/*
 * the launchPlugin() extern method allows for dynamic loading and launching
 * of this plugin
 */

extern "C"
{
    plugin* launchPlugin() { return new mod_template; }
}

mod_template::mod_template()
{
    templateWin = NULL;
}


mod_template::~mod_template()
{
    /* delete the window if it exists. */
    if ( templateWin != NULL )
    {
        delete templateWin;
    }
}


 /* when the user wants to create an instance of this plugin, the client loads
  * the .so, creates an instance of the class and then calls exec()
  *
  * exec() needs to start the GUI portion(s) of the plugin. its the main() of
  * the plugin.
  *
  * exec() receives a pointer to the info on the host we are connected to
  * a numerical ID unique to this instance of a plugin (needed for networking) and a
  * pointer to the module reference we were launched from. only the numerical ID is
  * significant to the plugin subclass (it is needed to create netmessages). the
  * other two pararmeters passed should be passed to init(...).
  */

void mod_template::exec(hostObj *_host, unsigned int _moduleID, moduleList *_modList)
{
    /*
     * init(...) MUST be called before ANYTHING ELSE occurs EXACTLY as shown below
     * failure to do this will result in the plugin doing Bad Things(tm)
     *
     * after this call, you can reference _moduleID as moduleID
     */
    init(_host, _moduleID, _modList);

    /*
     * now we set up our GUI by creating a templateWin class
     * this is not the only way of doing it, but the most common.
     * alternatively one could use a different GUI element at plugin launch,
     * use no GUI at all, or delay creation of a GUI element until something else
     * occurs (such as network communication)
     */
    templateWin = new mod_templatewin(moduleID, this, "Template Module");
    templateWin->show();
}


 /*
  * recv() gets called when a netmessage arrives over the network for this plugin.
  * it is handed both a pointer to the netmessage as well as the
  * numerical ID defining the TYPE of netmessage. these IDs can be referenced in
  * olympus/src/common/commid.h
  *
  * usually, to do anything USEFUL with the netmessage one needs to create a pointer
  * to an object of the specific type of netmessage you've received and cast the pointer
  * called 'message' to that type. (see recv(...) below and docs/README-netmessage.txt)
  */

void mod_template::recv(netmessage *message, int objectID)
{
    /*
    if (objectID == COMMAND_SomeCommand)
    {
        nmSomeCommand *thisCommand = (nmSomeCommand*) message;
    }
    else if (objectID == COMMAND_SomeOtherCommand)
    {
        nmSomeOtherCommand *thisOtherCommand = (nmSomeOtherCommand*) message;
    }
    ...and so on.
    */
}


 /*
  * getFocus() gets called when the user selects a module that is already
  * running from the main client gui. getFocus() should then make sure that all
  * and any GUI components are brought to the front and any interupted processing
  * is continued
  */

void mod_template::getFocus()
{
    templateWin->show();
    templateWin->setActiveWindow();
    templateWin->raise();
}


 /*
  * mod_templatewin:  an example of a simple GUI usable by a plugin. It has no real
  * functionality beyond setting up its environment properly and printing
  * "Hello, Olympus!" into a dialog box. Useful as a template, though.
  */

mod_templatewin::mod_templatewin( unsigned int _moduleID, mod_template *_parentPlugin, char *name)
    : QDialog(0,name)
{
    /*
     * we store the moduleID to give to netmessages when we create them to communicate
     * with the server.
     *
     * the moduleID is used to differentiate the traffic from this plugin from
     * traffic from other plugins or the client itself over the network
     */
    moduleID = _moduleID;

    /*
     * using this pointer to the plugin subclass, this window class can communicate
     * via the network, get host information, save/restore information via the registry,
     * and a number of other things.
     */
    parentPlugin = _parentPlugin;

    setCaption(parentPlugin->hostCommonName() + ": Olympus Template");

    QLabel * hello = new QLabel(this, "Hello");
    hello->setGeometry(0, 0, 300, 50);
    hello->setText( " Hello, Olympus! " );

    setFixedSize(300,50);
}


mod_templatewin::~mod_templatewin()
{
    /*
     * clean up whatever you need to here.
     */
}


void mod_templatewin::closeEvent( QCloseEvent *e )
{
    /* accept the event to close the window */
    e->accept();

    /*
     * and now delete the plugin itself
     * this assumes that by closing the window, the plugin should finish
     * while this is the case with nearly all plugins, it may not be universal
     */
    delete parentPlugin;
}
